import type { FieldType } from "./models"

export interface Profile {
  lastUpdatedBy: "Dropbox"
  /** Unix seconds */
  updatedAt: number
  profileName: string
  salt: string // base64
  masterKey: string // base64
  iterations: number // 50000
  uuid: string // 32 chars
  overviewKey: string // "b3B...IMO52D"
  createdAt: number // Unix seconds
}

export interface EncryptedItem {
  category: string // "001"
  /** Unix seconds */
  created: number
  d: string // "b3BkYXRhMbt"
  folder: string // 32 chars
  hmac: string // base64
  k: string // base64
  o: string // base64
  tx: number // Unix seconds
  updated: number // Unix seconds
  uuid: string // 32 chars
}

export type TextField = {
  type: FieldType.Text
  value: string
  designation: string
  name: string
}
export type BooleanField = {
  type: FieldType.Checkbox
  name: string
  value?: "✓" | string
}

export type ItemField =
  | TextField
  | BooleanField
  | {
      // @TODO: This currently catches all item fields.
      type: FieldType
      value: string
      designation?: string
      name: string
    }

declare namespace ItemSection {
  type A = {
    guarded: "yes"
    clipboardFilter?: string
  }

  type String = {
    k: "string"
    v: string
    /** Unique name */
    n: string // "firstname" | "initial" | "address" | "class" | "conditions" | "expiry_date"
    a?: A
    /** User-readable title */
    t: string // "first name" | "initial" | "address" | "license class" | "conditions / restrictions" | "expiry date"
  }
  type Menu = {
    k: "menu"
    v: string // "female"
    a: A
    t: string // "sex"
  }
  type Date = {
    k: "date"
    v: number // 359100000
    n: string // "birthdate"
    a: A
    t: string // "birth date" | "date of birth"
  }
  type Gender = {
    k: "gender"
    n: "sex"
    v: string // "female"
    t: "sex"
  }
  type MonthYear = {
    k: "monthYear"
    n: string // "expiry_date"
    v: number // 2515
    t: string // "expiry date"
  }
  type Concealed = {
    k: "concealed"
    n: "password"
    v: string
    a?: {
      generate: "off"
    }
    t: "password"
  }

  type Any = String | Menu | Date | Gender | MonthYear | Concealed
}

// One of them is empty?, 0C4F27910A64488BB339AED63565D148
export interface Item {
  htmlForm?: {
    htmlAction: string // "/login/"
    htmlMethod: "post" | "get"
  }
  notesPlain?: string
  sections: {
    name: string // "name" | "title" | "internet"
    title: string // "Identification" | "Address", "Internet Details"
    fields?: ItemSection.Any[]
  }[]
  /** Web form fields */
  fields?: ItemField[]
}

export interface Band {
  [uuid: string]: EncryptedItem
}

export interface Folder {
  created: number // 1373754128
  overview: string // "b3BkYXRhT/../KBM="
  smart?: true
  tx: number // 1373754523
  updated: number // 1373754134
  uuid: string // "AC78552EB06A4F65BEBF58B4D9E32080"
}

export interface Overview {
  ps: number
  title?: string
  ainfo?: string
  url?: string
  tags?: string[]
  URLs?: { u: string }[]
  uuid?: string // Added manually
}
